/**
 * @Author: Caow
 * @Date: 2019-12-27 17:18:52
 */

import ImageryLayerFactory from './imagery/ImageryLayerFactory'
import TerrainFactory from './terrain/TerrainFactory'
import Viewer from './viewer/Viewer'
import {
  LayerGroup,
  GeoJsonLayer,
  HtmlLayer,
  LabelLayer,
  PrimitiveLayer,
  TilesetLayer,
  TopoJsonLayer,
  VectorLayer
} from './layer'

import {
  Billboard,
  Circle,
  DivIcon,
  Label,
  Point,
  Polyline,
  Polygon,
  Model,
  Tileset
} from './overlay'

import {
  area,
  bounds,
  mid,
  center,
  distance,
  heading,
  isBetween,
  parabola,
  curve
} from './math'

//新增加载
import { Water, Fire, Pipeline, FlyWithTime } from './effects'

const { Cesium } = BC.Namespace

Cesium.Math.area = area
Cesium.Math.bounds = bounds
Cesium.Math.mid = mid
Cesium.Math.center = center
Cesium.Math.distance = distance
Cesium.Math.heading = heading
Cesium.Math.isBetween = isBetween
Cesium.Math.parabola = parabola
Cesium.Math.curve = curve

const core = {
  ImageryLayerFactory,
  TerrainFactory,
  Viewer,
  World: Viewer,
  LayerGroup,
  GeoJsonLayer,
  HtmlLayer,
  LabelLayer,
  PrimitiveLayer,
  TilesetLayer,
  TopoJsonLayer,
  VectorLayer,
  Billboard,
  Circle,
  DivIcon,
  Label,
  Point,
  Polyline,
  Polygon,
  Model,
  Tileset,
  Math: Cesium.Math,
  //新增
  Water,
  Fire,
  Pipeline,
  FlyWithTime
}

BC.mixin(core)
